package hamster;

/**
 * Dieses BlueJ-Hamster-Projekt bietet den Hamstern die Mglichkeit, das so
 * genannte Nimm-Spiel zu spielen. Es ist ein Spiel fr zwei Hamster. Der
 * Standard-Hamster fungiert quasi als Schiedsrichter. Er markiert den
 * Ausgangspunkt eines Spiels. Vor ihm liegt eine Reihe mit Krnern, auf jeder
 * Kachel jeweils ein Korn. Bei Spielbeginn mssen sich die beiden spielenden
 * Hamster zunchst zum Standard-Hamster begeben. Dann mssen sie die
 * Krnerreihe ablaufen und abwechselnd jeweils entweder ein oder zwei Krner
 * nehmen. Sie wissen dabei, wie lang die Krnerreihe ist. Wer das letzte Korn
 * der Reihe nimmt, gewinnt das Spiel.
 * 
 * @author Dietrich Boles (Universitaet Oldenburg)
 * @version 1.0 (28.01.2006)
 * 
 */
public class NimmSpiel {

	private SpielHamster[] spieler; // speichert die spielenden Hamster

	private Hamster schiedsrichter; // Schiedsrichter

	/**
	 * Initialisierung eines NimmSpiels.
	 */
	public NimmSpiel() {
		// nichts zu tun
	}

	/**
	 * Spielen eines NimmSpiels mit zwei Hamstern
	 * 
	 * @param hamster1
	 *            Spieler A
	 * @param hamster2
	 *            Spieler B
	 */
	public void spielen(SpielHamster hamster1, SpielHamster hamster2) {
		this.spieler = new SpielHamster[2];
		this.spieler[0] = hamster1;
		this.spieler[1] = hamster2;
		this.schiedsrichter = Hamster.getStandardHamster();

		int laengeDerKoernerReihe = this.ermittleLaengeDerKoernerReihe();
		if (laengeDerKoernerReihe == 0) {
			this.schiedsrichter.schreib("Die Koernerreihe muss mindestens "
					+ "ein Korn lang sein!");
			return;
		}

		// Spieler zum Startpunkt holen und Laenge der Kornreihe mitteilen
		this.spieler[0].begibDichZumStart(this.schiedsrichter.getReihe(),
				this.schiedsrichter.getSpalte(), this.schiedsrichter
						.getBlickrichtung());
		this.spieler[1].begibDichZumStart(this.schiedsrichter.getReihe(),
				this.schiedsrichter.getSpalte(), this.schiedsrichter
						.getBlickrichtung());
		this.spieler[0].setLaengeDerKornReihe(laengeDerKoernerReihe);
		this.spieler[1].setLaengeDerKornReihe(laengeDerKoernerReihe);

		// jetzt kann das Spiel losgehen
		int aktuellerIndex = 0;
		int gefressen = 0;
		while (true) {
			gefressen = this.spieler[aktuellerIndex]
					.naechsterSpielzug(gefressen);
			if (gefressen < 1 || gefressen > 2) {
				this.schiedsrichter.schreib("Spieler "
						+ (aktuellerIndex == 0 ? "A" : "B")
						+ " hat sich nicht an die Spielregeln "
						+ "gehalten und hat damit verloren!");
				return;
			}
			laengeDerKoernerReihe = laengeDerKoernerReihe - gefressen;
			if (laengeDerKoernerReihe <= 0) {
				this.schiedsrichter.schreib("Spieler "
						+ (aktuellerIndex == 0 ? "A" : "B") + " hat gewonnen!");
				return;
			}

			// bewirkt, dass die beiden Hamster abwechselnd einen
			// Spielzug ausfuehren
			aktuellerIndex = (aktuellerIndex + 1) % 2;
		}
	}

	/**
	 * Spielen eines NimmSpiels mit zwei Hamstern. Vorher wird ein
	 * vordefiniertes Territorium geladen.
	 * 
	 * @param dateiName
	 *            Name einer Datei mit einem vordefinierten Territorium
	 * @param hamster1
	 *            Spieler A
	 * @param hamster2
	 *            Spieler B
	 */
	public void spielen(String dateiName, SpielHamster hamster1,
			SpielHamster hamster2) {
		Territorium.ladeTerritorium(dateiName);
		Hamster ham = Hamster.getStandardHamster();
		hamster1.init(ham.getReihe(), ham.getSpalte(), ham.getBlickrichtung(),
				ham.getAnzahlKoerner());
		hamster2.init(ham.getReihe(), ham.getSpalte(), ham.getBlickrichtung(),
				ham.getAnzahlKoerner());
		this.spielen(hamster1, hamster2);
	}

	private int ermittleLaengeDerKoernerReihe() {
		int anzahl = 0;
		int schritte = 0;
		while (this.schiedsrichter.vornFrei()) {
			this.schiedsrichter.vor();
			schritte = schritte + 1;
			if (!this.schiedsrichter.kornDa()) {
				break;
			}
			anzahl = anzahl + 1;
		}

		// zurueck laufen
		this.schiedsrichter.linksUm();
		this.schiedsrichter.linksUm();
		for (int i = 0; i < schritte; i++) {
			this.schiedsrichter.vor();
		}
		this.schiedsrichter.linksUm();
		this.schiedsrichter.linksUm();

		return anzahl;
	}
}
